package org.alltimeflashdreamer.log4jmonitor;

import java.awt.BorderLayout;
import java.text.MessageFormat;
import java.util.Date;
import javax.swing.BorderFactory;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import org.apache.log4j.Logger;
import org.apache.log4j.spi.LoggingEvent;

/**
 * Title: DetailPanel<br>
 * Description: This class represents a panel for showing a stack trace.<br>
 * This code has been copied from the authors below.
 *
 * @author    <a href="mailto:chainsaw@puppycrawl.com">Oliver Burn</a>, revised by
 *            <a href="mailto:florian@steinsiepe.ch">Florian Steinsiepe</a>
 * @version   1.1
 */

class DetailPanel extends JPanel implements ListSelectionListener
{
    /** used to log events */
    private final static Logger LOG = Logger.getLogger( DetailPanel.class );

    /** used to format the logging event */
    private final static MessageFormat FORMATTER = new MessageFormat(
            "<b>Time:</b> <code>{0,time,medium}</code>" +
            "&nbsp;&nbsp;<b>Level:</b> <code>{1}</code>" +
            "&nbsp;&nbsp;<b>Thread:</b> <code>{2}</code>" +
            "&nbsp;&nbsp;<b>NDC:</b> <code>{3}</code>" +
            "<br><b>Logger:</b> <code>{4}</code>" +
            "<br><b>Location:</b> <code>{5}</code>" +
            "<br><b>Message:</b>" +
            "<pre>{6}</pre>" +
            "<b>Throwable:</b>" +
            "<pre>{7}</pre>" );

    /** the model for the data to render * */
    private final MyTableModel model;
    /** pane for rendering detail * */
    private final JEditorPane detailsPane;

    /**
     * Creates a new <code>DetailPanel</code> instance.
     *
     * @param table  the JTable to listen for selections on
     * @param model  the model backing the table
     */
    DetailPanel( JTable table, final MyTableModel model )
    {
        this.model = model;
        setLayout( new BorderLayout() );
        setBorder( BorderFactory.createTitledBorder( "Details: " ) );

        detailsPane = new JEditorPane();
        detailsPane.setEditable( false );
        detailsPane.setContentType( "text/html" );
        add( new JScrollPane( detailsPane ), BorderLayout.CENTER );

        table.getSelectionModel().addListSelectionListener( this );
    }

    /**
	 * Updates the content of the details pane with the message of
	 * the "latest" selected event in the table.
	 *
     * @param event  the event to react to
     * @see javax.swing.event.ListSelectionListener
	 * @see javax.swing.event.ListSelectionEvent
     */
    public void valueChanged( ListSelectionEvent event )
    {
        //Ignore extra messages.
        if ( event.getValueIsAdjusting() )
            return;

        final ListSelectionModel lsm = ( ListSelectionModel )event.getSource();
        if ( lsm.isSelectionEmpty() )
        {
            detailsPane.setText( "Nothing selected" );
        }
        else
        {
            final int selectedRow = lsm.getMinSelectionIndex();
            final LoggingEvent e = model.getLoggingEvent( selectedRow );
            final Object[] args =
            {
                new Date( e.timeStamp ),
                e.level,
                e.getThreadName(),
                e.getNDC(),
                e.categoryName,
                ( e.getLocationInformation() == null ) ? null : e.getLocationInformation().fullInfo,
                e.getMessage(),
                getThrowableStrRep( e )
            };
            detailsPane.setText( FORMATTER.format( args ) );
            detailsPane.setCaretPosition( 0 );
        }
    }

    /**
     * Returns a string representation of a throwable.
     *
     * @param event   contains the throwable information
     * @return        a <code>String</code> value
	 * @see org.apache.log4j.spi.LoggingEvent
     */
    private static String getThrowableStrRep( LoggingEvent event )
    {
        final String[] strs = event.getThrowableStrRep();
        if ( strs == null )
            return null;

        final StringBuffer sb = new StringBuffer();
        for ( int i = 0; i < strs.length; i++ )
            sb.append( strs[i] ).append( "\n" );

        return sb.toString();
    }
}
