package org.alltimeflashdreamer.log4jmonitor;

import java.awt.*;
import java.io.IOException;
import java.util.*;
import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.table.*;
import org.apache.log4j.Level;

/**
 * Title: LogPanel<br>
 * Description: This is the main GUI panel which contains the components that show
 * the received logging messages to the user.<br>
 * This code has been copied from the authors below.
 *
 * @author    Copyright (c) 2001 by Sven Mosimann, Adrian Seiler, Florian Steinsiepe of EcoLogic GmbH,
 *            revised 2002 by <a href="mailto:florian@steinsiepe.ch">Florian Steinsiepe</a>
 * @version   1.1
 */

public class LogPanel extends JPanel
{
    /** name of property for port name * */
    public final static String PORT_PROP_NAME = "log4jmonitor.port";
    /** color for the selection foreground of the JTable */
    private final static Color COLOR_SELECTED = new Color( 227, 227, 239 );
	/** color for debug text within a table cell */
	private final static Color COLOR_DEBUG = Color.black;
	/** color for info text within a table cell */
	private final static Color COLOR_INFO = new Color( 0, 0, 220 );
	/** color for warn text within a table cell */
	private final static Color COLOR_WARN = new Color( 180, 0, 180 );
	/** color for error text within a table cell */
	private final static Color COLOR_ERROR = new Color( 255, 100, 100 );
	/** color for fatal text within a table cell */
	private final static Color COLOR_FATAL = Color.red;
	/** color for off text within a table cell */
	private final static Color COLOR_OFF = Color.lightGray;
	/** font to use within table cells and for "cell content measurements" */
	private final static Font FONT_TABLECELL = new Font( "Dialog", Font.PLAIN, 12 );
    /** the default port number to listen on * */
    private final static int DEFAULT_PORT = 27272;
    /** THICKNESS of the border */
    final int BORDERTHICKNESS = 5;
    /** create the all important model */
    final MyTableModel model = new MyTableModel();
    /** Add control panel */
    final ControlPanel cp = new ControlPanel( model );
    /** Create the table */
    final JTable table = new JTable( model );
    /** Contains the table */
    final JScrollPane scrollPane = new JScrollPane( table );
    /** Create the details */
    final JPanel details = new DetailPanel( table, model );

	/** the (often used) font metrics to measure string widths */
	private FontMetrics fontMetrics;
    /** Add the table and stack trace into a splitter */
    private JSplitPane jsp;
    /** Border Layout */
    private BorderLayout borderLayout1 = new BorderLayout();
    /** Line Border */
    private Border lineBorder;
    /** get the backgroundcolor of the panel */
    private Color clBackground;
	/** list of column widths to use */
	private int[] iColumnWidths;


    /** Constructor initializes the list of column widths and builds up the GUI. */
    public LogPanel()
    {
		iColumnWidths = new int[MyTableModel.COL_NAMES.length];
		Arrays.fill( iColumnWidths, 0 );

        try
        {
            jbInit();
        } catch ( Exception ex )
        {
            ex.printStackTrace();
        }
    }


    /**
     * Calculates the table column width of the different columns. The goal is to have as much
	 * space for the "Logger" and the "Message" columns as possible. Therefore, the other
	 * columns are made as small as possible, depending on their content's string width (for
	 * boolean columns, the column title is relevant).
     */
    public void setColumnWidths()
    {
		TableColumn column;
        for ( int i = 0; i < table.getColumnModel().getColumnCount(); i++ )
        {
            column = table.getColumnModel().getColumn( i );
			if ( !( column.getCellRenderer() instanceof MyCellRenderer ) )
			{
				column.setCellRenderer( new MyCellRenderer() );
				column.setMinWidth( fontMetrics.stringWidth( column.getHeaderValue().toString() ) );
				column.setResizable( true );
			}

			//get header content width and update the internal list, if necessary
			int iColumnHeaderWidth = fontMetrics.stringWidth( column.getHeaderValue().toString() );
			if ( iColumnHeaderWidth > iColumnWidths[i] )
				iColumnWidths[i] = iColumnHeaderWidth;

			//finally set width of column
			int iNewColumnWidth = 0;
			if ( !column.getHeaderValue().toString().equals( MyTableModel.COL_NAMES[4] ) )
				iNewColumnWidth = iColumnWidths[i];
			else
			{
				for ( int j = 0; j < iColumnWidths.length; j++ )
				{
					if ( i != j )
					   iNewColumnWidth += iColumnWidths[j];
				}
			}
			column.setPreferredWidth( iNewColumnWidth );
			//column.setWidth( iNewColumnWidth );
        }

		//hard reset of the internal list valuea
		Arrays.fill( iColumnWidths, 0 );
    }

    /**
     * Builds up the GUI and its components.
     *
     * @exception Exception  wrapper for each problem that may occur
     */
    private void jbInit()
        throws Exception
    {
        //set up layout of LogPanel (Control Panel and Splitpanel)
        this.setLayout( borderLayout1 );
        clBackground = this.getBackground();
		fontMetrics = this.getFontMetrics( FONT_TABLECELL );
        table.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		table.setFont( FONT_TABLECELL );

        lineBorder = BorderFactory.createLineBorder( clBackground, BORDERTHICKNESS );
        cp.setBorder( lineBorder );
        this.add( cp, BorderLayout.NORTH );

        scrollPane.setBorder( BorderFactory.createTitledBorder( "Events: " ) );
        scrollPane.setPreferredSize( new Dimension( 900, 300 ) );
        scrollPane.setMinimumSize( new Dimension( 200, 200 ) );
        details.setPreferredSize( new Dimension( 900, 300 ) );
        details.setMinimumSize( new Dimension( 200, 100 ) );
        jsp = new JSplitPane( JSplitPane.VERTICAL_SPLIT, scrollPane, details );
        jsp.setBorder( lineBorder );
        add( jsp, BorderLayout.CENTER );
        jsp.setDividerLocation( 0.7 );

        //start SocketServer
        setupReceiver( model );
    }


    /**
     * Setup the messages receiving engine.<br>
	 * Because the server port (on which the applications listens the log4j socket appenders)
	 * may be defined through the JVM parameter "log4jmonitor.port", this parameter is
	 * checked first. If it isn't defined, the application's default port (27272) is used.
     *
     * @param model  the MyTableModel instance to get the values from
     */
    private void setupReceiver( MyTableModel model )
    {
        int port = DEFAULT_PORT;
        final String strRep = System.getProperty( PORT_PROP_NAME );
        if ( strRep != null )
        {
            try
            {
                port = Integer.parseInt( strRep );
            } catch ( NumberFormatException nfe )
            {
                JOptionPane.showMessageDialog( this, "Unable to parse port number from '" + strRep +
				        "', quitting.", "Log4JMonitor", JOptionPane.ERROR_MESSAGE );
                System.exit( 1 );
            }
        }

        try
        {
            final LoggingReceiver lr = new LoggingReceiver( model, port );
            lr.start();
        } catch ( IOException e )
        {
            JOptionPane.showMessageDialog( this, "Unable to create socket on port " + port + ", quitting.",
                    "Log4JMonitor", JOptionPane.ERROR_MESSAGE );
            System.exit( 1 );
        }
    }

//------------------------------ inner class MyCellRenderer --------------------------//
    /**
	 * Title: MyCellRenderer<br>
	 * Description: This small class renders the table cells according to the data
	 * of the different log message attributes and the log level.
	 *
	 * @author Copyright (c) 2002 by <a href="mailto:florian@steinsiepe.ch">Florian Steinsiepe</a>
	 * @version 1.1
	 */
    private class MyCellRenderer extends DefaultTableCellRenderer
    {
		private MyBooleanCell mbc;
		private final int INSETS = 10;

        /**
         * Override of superclass method to set the colors correctly, depending on
         * the log level of the message that is shown in the current table row.
         *
         * @param table       the table to render the cells of
         * @param value       the current value (as Object) to be rendered
         * @param isSelected  whether the cell is selected or not
         * @param hasFocus    whether the cell has the (input) focus or not
         * @param row         the row number
         * @param column      the column number
         * @return            the rendered table cell component
         */
        public Component getTableCellRendererComponent( JTable table, Object value,
                boolean isSelected, boolean hasFocus, int row, int column )
        {
            //let the master do the job first...
            super.getTableCellRendererComponent( table, value, isSelected, hasFocus, row, column );

			//do special if Boolean / check box cell
            if ( value instanceof Boolean )
			{
                mbc = new MyBooleanCell( ( Boolean )value, isSelected );
				iColumnWidths[column] = mbc.getWidth() + INSETS;
				return mbc;
			}

            //find out the descriptor of the current entry's priority setting
			setFont( FONT_TABLECELL );
            String sLevel = "";
            for ( int i = 0; i < table.getColumnCount(); i++ )
            {
                if ( table.getColumnName( i ).equals( MyTableModel.COL_NAMES[1] ) )
                {
                    sLevel = ( ( Level )table.getValueAt( row, i ) ).toString();
                    break;
                }
            }

            //set foreground to priority-related color
            if ( sLevel.equals( "INFO" ) )
                setForeground( COLOR_INFO );
            else if ( sLevel.equals( "ERROR" ) )
                setForeground( COLOR_ERROR );
            else if ( sLevel.equals( "WARN" ) )
                setForeground( COLOR_WARN );
            else if ( sLevel.equals( "FATAL" ) )
                setForeground( COLOR_FATAL );
            else if ( sLevel.equals( "OFF" ) )
                setForeground( COLOR_OFF );
            else
                setForeground( COLOR_DEBUG );

            //return component to be painted
			int iStringWidth = fontMetrics.stringWidth( getText() );
			if ( iStringWidth > iColumnWidths[column] )
				iColumnWidths[column] = iStringWidth + INSETS;
            return this;
        }

		//------------------------------ inner class MyBooleanCell --------------------------//
		/**
		 * Title: MyBooleanCell<br>
		 * Description: Renders a cell that has the column class parameter Boolean.class
		 * by showing a check box that is either selected or not, depending on the value.
		 *
		 * @author Copyright (c) 2002 by <a href="mailto:florian@steinsiepe.ch">Florian Steinsiepe</a>
		 * @version 1.1
		 */
        private class MyBooleanCell extends JCheckBox
        {
			/**
			 * Sets alignment, selection and background color.
			 *
			 * @param value         the value object
			 * @param bIsSelected   whether the cell is selected within the whole table row
			 */
            private MyBooleanCell( Boolean value, boolean bIsSelected )
            {
                this.setHorizontalAlignment( JLabel.CENTER );
                this.setSelected( value.booleanValue() );
                this.setBackground( ( bIsSelected ) ? COLOR_SELECTED : Color.white );
            }
        }
    }
}
