package org.alltimeflashdreamer.log4jmonitor;

import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.SortedSet;
import java.util.TreeSet;
import javax.swing.table.AbstractTableModel;
import org.apache.log4j.Logger;
import org.apache.log4j.Level;
import org.apache.log4j.spi.LoggingEvent;

/**
 * Title: MyTableModel<br>
 * Description: Represents a list of <code>LoggingEvent</code> objects that are sorted
 * on logging time. Methods are provided to filter the events that are visible.<br>
 * This code has been copied from the authors below.
 *
 * @author    <a href="mailto:chainsaw@puppycrawl.com">Oliver Burn</a> , revised
 *      by <a href="mailto:florian@steinsiepe.ch">Florian Steinsiepe</a>
 * @version   1.1
 */

class MyTableModel extends AbstractTableModel
{
    /** names of the columns in the table */
    final static String[] COL_NAMES = {"Time", "Level", "Trace", "Logger", "Message"};
    /** used to log own messages */
    private final static Logger LOG = Logger.getLogger( MyTableModel.class );
    /** use the compare logging events */
    private final static Comparator MY_COMP =
        new Comparator()
        {
            /**
             * Compares two LoggingEvents (received as objects, as defined in the
             * Comparator interface) to allow a sorting of the whole list of those
             * events.
             *
             * @param aObj1  the first object/event to compare
             * @param aObj2  the second object/event to compare
             * @return       -1, if aObj1 is less, 0 if equal and 1 if aObj1 is higher
             *      (within the sorting priority)
             * @see          java.util.Comparator
             */
            public int compare( Object aObj1, Object aObj2 )
            {
                if ( ( aObj1 == null ) && ( aObj2 == null ) )
                {
                    //if both objects are null, treat them as equal
                    return 0;
                }
                else if ( aObj1 == null )
                {
                    //if the first object is null, treat it as less (than everything)
                    return -1;
                }
                else if ( aObj2 == null )
                {
                    //same as above, but vice versa
                    return 1;
                }

                //assuming only the given objects are LoggingEvents only
                //and compare their time stamps
                final LoggingEvent le1 = ( LoggingEvent )aObj1;
                final LoggingEvent le2 = ( LoggingEvent )aObj2;
                if ( le1.timeStamp < le2.timeStamp )
                    return 1;
                else if ( le1.timeStamp > le2.timeStamp )
                    return -1;
                else
                    return 0;
            }
        };
    /** definition of an empty list */
    private final static LoggingEvent[] EMPTY_LIST = new LoggingEvent[]{};
    /** used to format dates */
    private final static DateFormat DATE_FORMATTER =
            DateFormat.getDateTimeInstance( DateFormat.SHORT, DateFormat.MEDIUM );

    /** the lock to control access */
    private final Object oLock = new Object();
    /** set of all logged events - not filtered */
    private final SortedSet allSortedEvents = new TreeSet( MY_COMP );
    /** events that are visible after filtering */
    private LoggingEvent[] filteredEvents = EMPTY_LIST;
    /** list of events that are buffered for processing */
    private final List pendingEvents = new ArrayList();
    /** indicates whether event collection is paused to the UI */
    private boolean bIsPaused = false;
    /** filter for the thread */
    private String sThreadFilter = "";
    /** filter for the message */
    private String sMessageFilter = "";
    /** filter for the logger */
    private String sLoggerFilter = "";
    /** filter for the level */
    private Level levelFilter = Level.DEBUG;


    /** Creates a new <code>MyTableModel</code> instance. */
    MyTableModel()
    {
        final Thread t = new Thread( new Processor() );
        t.setDaemon( true );
        t.start();
    }


////////////////////////////////////////////////////////////////////////////
// TableModel Methods
////////////////////////////////////////////////////////////////////////////

    /**
     * Returns the (current) amount of rows, each containing on LoggingEvent.
     *
     * @return   current row count?
     * @see      javax.swing.table.TableModel
     */
    public int getRowCount()
    {
        synchronized ( oLock )
        {
            return filteredEvents.length;
        }
    }

    /**
     * Returns the (current) amount of columns, each containing one attribute of
     * the LoggingEvents.
     *
     * @return   current column count?
     * @see      javax.swing.table.TableModel
     */
    public int getColumnCount()
    {
        // does not need to be synchronized
        return COL_NAMES.length;
    }

    /**
     * Returns the name of the specified column.<br>
     * Note: No exception handling is implemented for invalid column indexes.
     *
     * @param iColumnIndex  the index of the column, starting with 0
     * @return              the name of the specified column
     * @see                 javax.swing.table.TableModel
     */
    public String getColumnName( int iColumnIndex )
    {
        // does not need to be synchronized
        return COL_NAMES[iColumnIndex];
    }

    /**
     * Returns the class of the specified column. This class is always "Object" except
     * for the column with index 2 ("Trace" column).<br>
     * Note: No exception handling is implemented for invalid column indexes.
     *
     * @param iColumnIndex  the index of the column, starting with 0
     * @return              the class of the specified column
     * @see                 javax.swing.table.TableModel
     */
    public Class getColumnClass( int iColumnIndex )
    {
        // does not need to be synchronized
        return ( iColumnIndex == 2 ) ? Boolean.class : Object.class;
    }

    /**
     * Returns the object within the table having the given coordinates.<br>
     * Note: No exception handling is implemented for invalid row indexes; invalid
     * column indexes are treated as the index of the error message column.
     *
     * @param iRowIndex     the index of the row, starting with 0
     * @param iColumnIndex  the index of the column, starting with 0
     * @return              the object stored at the given coordinates
     * @see                 javax.swing.table.TableModel
     */
    public Object getValueAt( int iRowIndex, int iColumnIndex )
    {
        synchronized ( oLock )
        {
            LoggingEvent event = filteredEvents[iRowIndex];

            switch ( iColumnIndex )
            {
                case 0:
                    //format object into date string
                    return DATE_FORMATTER.format( new Date( event.timeStamp ) );
                case 1:
                    //return the name of the event's priority
                    return event.level;
                case 2:
                    //if the event contains a "root cause", return true, else false
                    return ( event.getThrowableInformation() == null )
                             ? Boolean.FALSE : Boolean.TRUE;
                case 3:
                    //return the category name of the event
                    return event.categoryName;
                default:
                    //return the event's message, regardless of the column index
                    return event.getMessage();
            }
        }
    }

////////////////////////////////////////////////////////////////////////////
// Other Public Methods
////////////////////////////////////////////////////////////////////////////

    /**
     * Sets the level to filter events on. Only events of an equal or higher level
     * are now displayed.
     *
     * @param level     the level to filter on
     * @see             org.apache.log4j.Level
     */
    public void setLevelFilter( Level level )
    {
        synchronized ( oLock )
        {
            levelFilter = level;
            updateFilteredEvents( false );
        }
    }

    /**
     * Set the filter for the thread field.
     *
     * @param str  the string to match
     */
    public void setThreadFilter( String str )
    {
        synchronized ( oLock )
        {
            sThreadFilter = str.trim();
            updateFilteredEvents( false );
        }
    }

    /**
     * Set the filter for the message field.
     *
     * @param str  the string to match
     */
    public void setMessageFilter( String str )
    {
        synchronized ( oLock )
        {
            sMessageFilter = str.trim();
            updateFilteredEvents( false );
        }
    }

    /**
     * Set the filter for the logger field.
     *
     * @param str  the string to match
     */
    public void setLoggerFilter( String str )
    {
        synchronized ( oLock )
        {
            sLoggerFilter = str.trim();
            updateFilteredEvents( false );
        }
    }

    /**
     * Add a LoggingEvent to the list.
     *
     * @param event  a <code>LoggingEvent</code> value
     */
    public void addEvent( LoggingEvent event )
    {
        synchronized ( oLock )
        {
            pendingEvents.add( event );
        }
    }

    /** Clear the list of all events. */
    public void clear()
    {
        synchronized ( oLock )
        {
            allSortedEvents.clear();
            filteredEvents = new LoggingEvent[0];
            pendingEvents.clear();
            fireTableDataChanged();
        }
    }

    /**
     * Toggle collecting state (switch from "collecting" to "pausing" and vice versa).
     */
    public void toggle()
    {
        synchronized ( oLock )
        {
            bIsPaused = !bIsPaused;
        }
    }

    /**
     * @return   whether currently paused collecting events
     */
    public boolean isPaused()
    {
        synchronized ( oLock )
        {
            return bIsPaused;
        }
    }

    /**
     * Get the throwable information at a specified row in the filtered events.<br>
     * Note: No exception handling is implemented for invalid row indexes.
     *
     * @param iRowIndex  Description of the Parameter
     * @return           the throwable information
     */
    public LoggingEvent getLoggingEvent( int iRowIndex )
    {
        synchronized ( oLock )
        {
            return filteredEvents[iRowIndex];
        }
    }

////////////////////////////////////////////////////////////////////////////
// Private methods
////////////////////////////////////////////////////////////////////////////

    /**
     * Update the filtered events data structure.
     *
     * @param bIsInsertedToFront  indicates whether events were added to front of
     *      the events. If true, then the current first event must still exist in
     *      the list after the filter is applied.
     */
    private void updateFilteredEvents( boolean bIsInsertedToFront )
    {
        final List filtered = new ArrayList();
        final Iterator it = allSortedEvents.iterator();
        while ( it.hasNext() )
        {
            final LoggingEvent event = ( LoggingEvent )it.next();
            if ( matchFilter( event ) )
                filtered.add( event );
        }

        final LoggingEvent lastFirst = ( filteredEvents.length == 0 ) ? null : filteredEvents[0];
        filteredEvents = ( LoggingEvent[] )filtered.toArray( EMPTY_LIST );

        if ( bIsInsertedToFront && lastFirst != null )
        {
            final int index = filtered.indexOf( lastFirst );
            if ( index < 1 )
            {
                LOG.warn( "updateFilteredEvents: In strange state!" );
                fireTableDataChanged();
            }
            else
                fireTableRowsInserted( 0, index - 1 );
        }
        else
            fireTableDataChanged();
    }

    /**
     * Returns whether an event matches the filters.
     *
     * @param event  the event to check for a match
     * @return       whether the event matches the current filter configuration
     * @see          org.apache.log4j.spi.LoggingEvent
     */
    private boolean matchFilter( LoggingEvent event )
    {
        if ( event.level.isGreaterOrEqual( levelFilter ) &&
                ( event.getThreadName().indexOf( sThreadFilter ) >= 0 ) &&
                ( event.categoryName.indexOf( sLoggerFilter ) >= 0 ) )
        {
            final String rm = event.getRenderedMessage();
            if ( rm == null )
            {
                //matches only if we have not filtering in place
                return ( sMessageFilter.length() == 0 );
            }
            else
            {
                return ( rm.indexOf( sMessageFilter ) >= 0 );
            }
        }

        //not matching by default
        return false;
    }

//-------------------------- inner class Processor ----------------------------//
    /**
     * Helper that actually processes incoming events.
     *
     * @author    <a href="mailto:chainsaw@puppycrawl.com">Oliver Burn</a> , revised
     *      by <a href="mailto:florian@steinsiepe.ch">Florian Steinsiepe</a>
     * @version   1.1
     * @see       java.lang.Runnable
     */
    private class Processor implements Runnable
    {
        /**
         * Loops infinitely, getting the events from the internal list of pending
         * events (always waits a second before listing the new events again). If
         * somehow updates/notifications need to be done, they're processed too.
         */
        public void run()
        {
            for ( ; ;  )
            {
                try
                {
                    Thread.sleep( 1000 );
                } catch ( InterruptedException ie )
                {
                    //simply ignore, but log them
                    LOG.warn( "Processor.run: Interrupted sleep!", ie );
                }

                synchronized ( oLock )
                {
                    //ignore new events if currently pausing
                    if ( bIsPaused )
                        continue;

                    //were events added to head?
                    boolean bToHead = true;
                    boolean bNeedUpdate = false;
                    final Iterator it = pendingEvents.iterator();
                    while ( it.hasNext() )
                    {
                        final LoggingEvent event = ( LoggingEvent )it.next();
                        allSortedEvents.add( event );
                        bToHead = bToHead && ( event == allSortedEvents.first() );
                        bNeedUpdate = bNeedUpdate || matchFilter( event );
                    }
                    pendingEvents.clear();

                    if ( bNeedUpdate )
                        updateFilteredEvents( bToHead );
                }
            }
        }
    }
}
